#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from zope.contentprovider.interfaces import IContentProvider, ContentProviderLookupError, BeforeUpdateEvent
from zope.location.interfaces import ILocation

# import packages
from chameleon.astutil import Symbol
from chameleon.tales import StringExpr
from pyams_utils.tales import ContextExprMixin
from pyramid_zope_request import PyramidPublisherRequest
from zope.contentprovider.tales import addTALNamespaceData


def render_content_provider(econtext, name):
    name = name.strip()

    context = econtext.get('context')
    request = econtext.get('request')
    if isinstance(request, PyramidPublisherRequest):
        request = request._request
    view = econtext.get('view')

    registry = request.registry
    provider = registry.queryMultiAdapter((context, request, view), IContentProvider, name=name)

    # provide a useful error message, if the provider was not found.
    if provider is None:
        raise ContentProviderLookupError(name)

    # add the __name__ attribute if it implements ILocation
    if ILocation.providedBy(provider):
        provider.__name__ = name

    # Insert the data gotten from the context
    addTALNamespaceData(provider, econtext)

    # Stage 1: Do the state update
    registry.notify(BeforeUpdateEvent(provider, request))
    provider.update()

    # Stage 2: Render the HTML content
    return provider.render()


class ProviderExpr(ContextExprMixin, StringExpr):
    """provider: TALES expression"""

    transform = Symbol(render_content_provider)
